unit hiInstallEl;

interface

uses Windows,Kol,Share,Debug,lUtils;

type
   ThiInstallEl = class(TDebug)
     private
      DList,ListEl,ListLink:PStrList;
      Info:TData;
      Action:Integer;
      ExtFilePath:string;
      Icon:PIcon;
      AltTab:string;
      NameHint:string;

      function GetStartDir():string;
      procedure FindFiles(var List:PStrList; const Dir:string;Ext:array of string);
      procedure ListDir(const Dir:string);
      procedure RemoveFromName(Name,fdir:PChar);
      procedure InstallEl(fn:PChar);
      procedure UnInstallEl(fn:PChar);
     public

      _prop_CreateInstallLst:boolean;
      _prop_LoadInstallLst:boolean;
      _prop_DelInstFiles:boolean;
      _prop_ForceInstall:boolean;
      _prop_ElementsDir:string;
      _prop_BackUpDir:string;
      _prop_Exclusion:string;
      _prop_DirComp:string;
      _prop_ExtractDir:string;
      _prop_PrefExtrComp:string;
      _data_InstFileName:THI_Event;
      _data_UnInstFileName:THI_Event;
      _data_DefaultPath:THI_Event;
      _data_FilePath:THI_Event;
      _data_ElementsDir:THI_Event;
      _data_BackUpDir:THI_Event;
      _data_Tab:THI_Event;
      _data_NameHiFiles:THI_Event;
      _data_Exclusion:THI_Event;
      _data_DirComp:THI_Event;
      _data_ExtractDir:THI_Event;
      _data_PrefExtrComp:THI_Event;
      _event_onIndexStopAction:THI_Event;
      _event_onListDir:THI_Event;
      _event_onListEl:THI_Event;
      _event_onTab:THI_Event;
      _event_onTabNames:THI_Event;
      _event_onLinkFiles:THI_Event;
      _event_onEndLinkFiles:THI_Event;

     destructor Destroy; override;
     procedure _work_doInstall(var _Data:TData; Index:word);
     procedure _work_doUninstall(var _Data:TData; Index:word);
     procedure _work_doCreateInstallLst(var _Data:TData; Index:word);
     procedure _work_doLoadInstallLst(var _Data:TData; Index:word);
     procedure _work_doDelInstFiles(var _Data:TData; Index:word);
     procedure _work_doForceInstall(var _Data:TData; Index:word);
     procedure _work_doExtractComp(var _Data:TData; Index:word);
     procedure _work_doTab(var _Data:TData; Index:word);
     procedure _work_doTabNames(var _Data:TData; Index:word);
     procedure _work_doLinkFiles(var _Data:TData; Index:word);
     procedure _work_doElementsDir(var _Data:TData; Index:word);
     procedure _work_doBackUpDir(var _Data:TData; Index:word);
     procedure _work_doExclusion(var _Data:TData; Index:word);
     procedure _work_doDirComp(var _Data:TData; Index:word);
     procedure _work_doExtractDir(var _Data:TData; Index:word);
     procedure _work_doPrefExtrComp(var _Data:TData; Index:word);
     procedure _var_InfoAction(var _Data:TData; Index:word);
     procedure _var_NameIcon(var _Data:TData; Index:word);
     procedure _var_NameHint(var _Data:TData; Index:word);
     procedure _var_AltTab(var _Data:TData; Index:word);
   end;

implementation

const
    NumExt = 6;
    ExtFiles: array[0..NumExt-1] of string = ('*.pas','*.ico','*.ini','*.h','*.cpp','*.html');

destructor ThiInstallEl.Destroy;
begin
  if assigned(Icon) then Icon.free;
  if assigned(DList) then DList.free;
  if assigned(ListEl) then ListEl.free;
  if assigned(ListLink) then ListLink.free;
  inherited;
end;

procedure ThiInstallEl._var_InfoAction;
begin
   _Data:= Info;
end;

procedure ThiInstallEl._work_doCreateInstallLst;
begin
   _prop_CreateInstallLst:= Readbool(_Data);
end;

procedure ThiInstallEl._work_doLoadInstallLst;
begin
   _prop_LoadInstallLst:= Readbool(_Data);
end;

procedure ThiInstallEl._work_doDelInstFiles;
begin
   _prop_DelInstFiles:= Readbool(_Data);
end;

procedure ThiInstallEl._work_doForceInstall;
begin
   _prop_ForceInstall:= Readbool(_Data);
end;

procedure ThiInstallEl._work_doElementsDir;
begin
   _prop_ElementsDir:= ToString(_Data);
end;

procedure ThiInstallEl._work_doBackUpDir;
begin
   _prop_BackUpDir:= ToString(_Data);
end;

procedure ThiInstallEl._work_doExclusion;
begin
   _prop_Exclusion:= ToString(_Data);
end;

procedure ThiInstallEl._work_doDirComp;
begin
   _prop_DirComp:= ToString(_Data);
end;

procedure ThiInstallEl._work_doExtractDir;
begin
   _prop_ExtractDir:= ToString(_Data);
end;

procedure ThiInstallEl._work_doPrefExtrComp;
begin
   _prop_PrefExtrComp:= ToString(_Data);
end;

procedure ThiInstallEl._var_NameIcon;
begin
   _Data.Data_type := data_icon;
   _Data.idata := integer(Icon);
end;

procedure ThiInstallEl._var_NameHint;
begin
   _Data.Data_type := data_str;
   _Data.sdata := NameHint;
end;

procedure ThiInstallEl._var_AltTab;
begin
   _Data.Data_type := data_str;
   _Data.sdata := AltTab;
end;

procedure ThiInstallEl.GetStartDir;
Var  Reg:HKey;
     DefaultPath:string;
     dt:TData;
begin
  dt.data_type:= data_null;
  DefaultPath:= ReadString(dt,_data_DefaultPath,'');
  if DefaultPath <> '' then
     begin
        Result:= DefaultPath + '\';
        Exit;
     end;
  Reg := RegKeyOpenRead(HKEY_CURRENT_USER,'Software\HiAsm');
  DefaultPath:= RegKeyGetStr(Reg,'InstallPath');
  if DefaultPath = '' then Result:= DefaultPath
  else Result:= DefaultPath + '\';
  RegKeyClose(Reg);
end;

procedure THIInstallEl.FindFiles;
var SRec:TSearchRec;
    n,i:cardinal;
    str:string;
begin
   for i:=0 to NumExt-1 do
      begin
         n:= FindFirst(Dir + Ext[i],faAnyFile,SRec);
         while n = 0 do
            begin
               List.Add(SRec.Name);
               n:= FindNext(SRec);
            end;
         FindClose(SRec);
      end;
end;

//#####################################################################
//#                                                                   #
//#                                                 #
//#                                                                   #
//#####################################################################

procedure THIInstallEl.ListDir;
var i:integer;
begin

  if (FileExists(Dir + 'Install.lst')) and  _prop_LoadInstallLst then DList.LoadFromFile(Dir + 'Install.lst')
  else
     begin
        FindFiles(DList,Dir,ExtFiles);
        if _prop_CreateInstallLst then DList.SaveToFile(Dir + 'Install.lst');
     end;

  i:=0;
  while i < DList.Count do
     begin
        if (DList.Items[i] = '.') or (DList.Items[i] = '..') or
        (DList.Items[i] = '') then
           begin
              DList.Delete(i);
              i:=i-1;
           end;
        i:= i+1;
     end;
end;



procedure THIInstallEl.RemoveFromName;
var list:PStrList;
    p,s:string;
    i,j:integer;
begin
  p := GetStartDir + 'Elements\' + fdir + '\Elements.ini';
  if not FileExists(p) then
     begin
        Info:= _doData(p);
        _hi_onEvent(_event_onIndexStopAction,1);
        Exit;
     end;
  list:= NewStrList;
  list.LoadFromFile(p);
  for i := 0 to list.Count - 1  do
   if pos(',',List.Items[i]) > 0 then
    begin
      s := LowerCase(List.Items[i]);
      if GetTok(s,',') = LowerCase(Name) then
       begin
         List.Delete(i);
         if _prop_DelInstFiles then
            begin
               if FileExists(GetStartDir + 'Elements\' + fdir + '\code\hi' + Name + '.pas') then
                  begin
                     DeleteFile(PChar(GetStartDir + 'Elements\' + fdir + '\code\hi' + Name + '.pas'));
                     DList.Add('hi' + Name + '.pas');
                  end;
               if FileExists(GetStartDir + 'Elements\' + fdir + '\code\hi' + Name + '.h') then
                  begin
                     DeleteFile(PChar(GetStartDir + 'Elements\' + fdir + '\code\hi' + Name + '.h'));
                     DList.Add('hi' + Name + '.h');
                  end;
               if FileExists(GetStartDir + 'Elements\' + fdir + '\code\hi' + Name + '.cpp') then
                  begin
                     DeleteFile(PChar(GetStartDir + 'Elements\' + fdir + '\code\hi' + Name + '.cpp'));
                     DList.Add('hi' + Name + '.cpp');
                  end;
               if FileExists(GetStartDir + 'Elements\' + fdir + '\icon\' + Name + '.ico') then
                  begin
                     DeleteFile(PChar(GetStartDir + 'Elements\' + fdir + '\icon\' + Name + '.ico'));
                     DList.Add(Name + '.ico');
                  end;
               if FileExists(GetStartDir + 'Elements\' + fdir + '\conf\' + Name + '.ini') then
                  begin
                     DeleteFile(PChar(GetStartDir + 'Elements\' + fdir + '\conf\' + Name + '.ini'));
                     DList.Add(Name + '.ini');
                  end;
               if FileExists(GetStartDir + 'Help\' + Name + '.html') then
                  begin
                     DeleteFile(PChar(GetStartDir + 'Help\' + Name + '.html'));
                     DList.Add(Name + '.html');
                  end;
            end;
       Break;
       end;
    end;
  list.SaveToFile(GetStartDir + 'Elements\' + fdir + '\Elements.ini');
  list.free;
end;

//#####################################################################
//#                                                                   #
//#                                               #
//#                                                                   #
//#####################################################################

procedure THIInstallEl.InstallEl;
var
   dt:TData;
   fl,list:PStrList;
   j,i,k,cnt:word;
   BDir,Dfc,Tfc,Bfc,NBfc,Tab,Hint,Name,fdir,p,nm:string;
begin
   dt.data_type:= data_null;
   BDir:= ReadString(dt,_data_BackUpDir,_prop_BackUpDir);
   ListDir(ExtractFilePath(fn));
   list:= NewStrList;
   fl := NewStrList;
   fl.LoadFromFile(fn);
   i:=0;
   while i < fl.Count do
      begin
         if (fl.Items[i] = '') then
            begin
               fl.Delete(i);
               i:=i-1;
            end;
         i:= i+1;
      end;

   j := 0;
   cnt := 0;
   while j < fl.Count do
      begin
         fdir := fl.Items[j];
         Name := GetTok(fdir,',');
         nm:= Name;
         Tab := GetTok(fdir,',');
         if pos(',',fdir) > 0 then Hint := GetTok(fdir,',')
         else
            begin
               Hint := fdir;
               fdir := Readstring(dt,_data_ElementsDir,_prop_ElementsDir);
            end;

      p := GetStartDir + 'Elements\' + fdir + '\Elements.ini';

      if not FileExists(p) then
         begin
            inc(j);
            Info:= _doData(p);
            _hi_onEvent(_event_onIndexStopAction,1);
            Continue;
         end;

      list.LoadFromFile(p);
      for i:= 0 to List.Count-1 do
         begin
         if Name + ',' = Copy(List.Items[i],1,Length(name)+1) then
            begin
               if _prop_ForceInstall then break;
               k:= 0;
               while k < DList.Count do
                  begin
                     if (Name = Copy(DList.Items[k],1,Length(Name))) or
                        ('hi' + Name = Copy(DList.Items[k],1,Length('hi' + Name))) then
                           begin
                              DList.Delete(k);
                              k:= k-1;
                           end;
                   k:=k+1;
                   end;
               Name:= '';
               break;
            end;
            end;
      if Name = '' then
         begin
            inc(j);
            Info:=_doData(nm);
            _hi_onEvent(_event_onIndexStopAction,2);
            Continue;
         end;
      if Name <> '' then ListEl.Add(Name);

      inc(cnt);
      i := 0;
      while i < list.Count do
         if copy(list.Items[i],1,Length(Tab)+1) = '+' + Tab then
            begin
               while (list.Items[i] <> '')and(i < list.Count) do inc(i);
               list.Insert(i,Name + ',' + Hint);
               i := 0;
               break;
            end
         else inc(i);
         if i = list.Count then
            begin
               i := 0;
               while List.Items[i] <> '+' do inc(i);
               while List.Items[i] <> '' do dec(i);
               list.Insert(i,'');
               list.Insert(i+1,'+' + Tab);
               list.Insert(i+2,Name + ',' + Hint);
            end;

      list.SaveToFile(p);
      inc(j);
   end;

   if cnt > 0 then
      begin
         for k:=0 to DList.Count - 1 do
            begin
               ForceDirectories(GetStartDir + 'Elements\' + fdir + '\' + BDir);
               ForceDirectories(GetStartDir + 'Elements\' + fdir + '\' + BDir + '\' + ExtractFileNameWOext(fn));
               Dfc:= ExtractFilePath(fn) + DList.Items[k];
               if ExtractFileExt(DList.Items[k]) = '.pas' then
                  Tfc:= GetStartDir + 'Elements\' + fdir + '\code\'
               else if ExtractFileExt(DList.Items[k]) = '.ico' then
                  Tfc:= GetStartDir + 'Elements\' + fdir + '\icon\'
               else if ExtractFileExt(DList.Items[k]) = '.ini' then
                  Tfc:= GetStartDir + 'Elements\' + fdir + '\conf\'
               else if ExtractFileExt(DList.Items[k]) = '.h' then
                  Tfc:= GetStartDir + 'Elements\' + fdir + '\code\'
               else if ExtractFileExt(DList.Items[k]) = '.cpp' then
                  Tfc:= GetStartDir + 'Elements\' + fdir + '\code\'
               else if ExtractFileExt(DList.Items[k]) = '.html' then
                  Tfc:= GetStartDir + 'Help\';
               Tfc:= Tfc + DList.Items[k];
               Bfc:= GetStartDir + 'Elements\' + fdir + '\' + BDir + '\' + ExtractFileNameWOext(fn)+ '\' + DList.Items[k];
               NBfc:= Bfc;
               while FileExists(NBfc) <> False do NBfc:= GetUniqueFilename(NBfc);
               CopyFile(PChar(Bfc),PChar(NBfc),true);
               DeleteFile(PChar(Bfc));
               CopyFile(PChar(Tfc),PChar(Bfc),true);
               DeleteFile(PChar(Tfc));
               CopyFile(PChar(Dfc),PChar(Tfc),true);
            end;
      end
   else DList.Clear;

   if DList.Count <> 0 then
      for i:=0 to DList.Count - 1 do _hi_onEvent(_event_onListDir,DList.Items[i]);
   if ListEl.Count <> 0 then
      for i:=0 to ListEl.Count - 1 do _hi_onEvent(_event_onListEl,ListEl.Items[i]);
   Info:= _doData(cnt);
   _hi_onEvent(_event_onIndexStopAction,3);

   list.free;
   fl.free;
end;

//#####################################################################
//#                                                                   #
//#                                             #
//#                                                                   #
//#####################################################################

procedure THIInstallEl.UnInstallEl;
var dt:TData;
    list:PStrList;
    i,j:integer;
    s,n:string;
begin
   dt.data_type:= data_null;
   list := NewStrList;
   list.LoadFromFile(fn);
   i:=0;
   while i < List.Count do
      begin
         if (List.Items[i] = '') then
            begin
               List.Delete(i);
               i:=i-1;
            end;
         i:= i+1;
      end;
   j:= 0;
   for i := 0 to list.Count-1 do
    begin
       s := List.Items[i];
       if pos(',',s) > 0 then
        begin
          n := gettok(s,',');
          RemoveFromName(PChar(n),PChar(s));
        end
       else RemoveFromName(PChar(s),PChar(ReadString(dt,_data_ElementsDir,_prop_ElementsDir)));
       ListEl.Add(s);
       j:=J+1;
    end;

   if DList.Count <> 0 then
      for i:=0 to DList.Count - 1 do _hi_onEvent(_event_onListDir,DList.Items[i]);
   if ListEl.Count <> 0 then
      for i:=0 to ListEl.Count - 1 do _hi_onEvent(_event_onListEl,ListEl.Items[i]);
   Info:= _doData(j);
   _hi_onEvent(_event_onIndexStopAction,3);
   list.Free;
end;


procedure ThiInstallEl._work_doInstall;
var fn:PChar;
    s:string;
begin
   s:= ReadString(_Data,_data_InstFileName,'');
   fn:= PChar(s);
   if s = '' then
      begin
         Info:= _doData('');
         _hi_onEvent(_event_onIndexStopAction,0);
         Exit;
      end;
   DList:= NewStrList;
   ListEl:= NewStrList;

   InstallEl(fn);

   DList.Free;
   ListEl.Free;
end;

procedure ThiInstallEl._work_doUninstall;
var fn:PChar;
    s:string;
begin
   s:= ReadString(_Data,_data_UnInstFileName,'');
   fn:= PChar(s);
   if s = '' then
      begin
         Info:= _doData('');
         _hi_onEvent(_event_onIndexStopAction,0);
         Exit;
      end;
   DList:= NewStrList;
   ListEl:= NewStrList;

   UnInstallEl(fn);

   DList.Free;
   ListEl.Free;
end;

//#####################################################################
//#                                                                   #
//#                   Elements.ini                #
//#                                                                   #
//#####################################################################

procedure ThiInstallEl._work_doTab;
var dt:TData;
    tab,p,fdir:string;
    epos,j,i,k,cnt:word;
    list:PStrList;
begin
   dt.data_type:= data_null;
   fdir:= ReadString(dt,_data_ElementsDir,_prop_ElementsDir);
   p := GetStartDir + 'Elements\' + fdir + '\Elements.ini';
   if not FileExists(p) then
      begin
         Info:= _doData(p);
         _hi_onEvent(_event_onIndexStopAction,1);
          Exit;
      end;
   list:= NewStrList;
   list.LoadFromFile(p);
   k:=0;
   for i:= 0 to list.Count - 1 do
      begin
         if Copy(List.Items[i],1,1) = '+' then
            begin
               epos:= pos('|', List.Items[i]);
               tab:= Copy(List.Items[i],2,epos-2);
               AltTab:= Copy(List.Items[i],epos+1,Length(List.Items[i])-epos);
               if tab <> '' then
                  begin
                     inc(k);
                     Info:= _doData(k);
                     _hi_onEvent(_event_onTab,tab);
                  end;
            end;
      end;

   list.free;
end;

//#####################################################################
//#                                                                   #
//#                                   #
//#                                                                   #
//#####################################################################

procedure ThiInstallEl._work_doTabNames;
var dt:TData;
    tab,p,Name,Hint,tabstr,fdir:string;
    epos,j,i,k,cnt:word;
    list:PStrList;
begin
   dt.data_type:= data_null;
   tab:=  ReadString(_Data,_data_Tab,'');
   fdir:= ReadString(dt,_data_ElementsDir,_prop_ElementsDir);
   if not assigned(Icon) then Icon:= NewIcon
   else Icon.Clear;

   p := GetStartDir + 'Elements\' + fdir + '\Elements.ini';
   if not FileExists(p) then
      begin
         Info:= _doData(p);
         _hi_onEvent(_event_onIndexStopAction,1);
          Exit;
      end;
   list:= NewStrList;
   list.LoadFromFile(p);
   k:= 0;
   i:= 0;
   while i < list.Count do
      if Copy(list.Items[i],1,Length(Tab)+1) = '+' + Tab then
         begin
            inc(i);
            while (list.Items[i] <> '') and (i < list.Count) do
               begin
                  if (list.Items[i] <> 'FPC+') and (list.Items[i] <> 'FPC-') and (Copy(list.Items[i],1,1) <> ';')then
                     begin
                        inc(k);
                        Info:= _doData(k);
                        epos:= pos(',', List.Items[i]);
                        Name:= Copy(List.Items[i],1,epos-1);
                        NameHint:= Copy(List.Items[i], epos+1, Length(List.Items[i])-epos);
                        Icon.LoadFromFile(GetStartDir + 'Elements\' + fdir + '\icon\' + Name + '.ico');
                        if Icon.Handle = 0 then Icon.LoadFromFile(GetStartDir + 'Elements\' + fdir + '\icon\' + 'nil.ico');
                        _hi_onEvent(_event_onTabNames, Name);
                        inc(i);
                     end
                  else inc(i);
               end;
         end
         else inc(i);
   list.free;
   Icon.Clear;
end;

//#####################################################################
//#                                                                   #
//#                         hi*.pas                        #
//#                                                                   #
//#####################################################################

procedure ThiInstallEl._work_doLinkFiles;
var dt:TData;
    fexcl,linkf,str,fn,p,fdir,fd:string;
    spos,epos,j,i,k,cnt:word;
    list,linklist:PStrList;
    notItem:boolean;
begin
   if not assigned(ListLink) then ListLink:= NewStrList
   else ListLink.Clear;
   fn:= ReadString(_Data,_data_NameHiFiles,'');
   if fn = '' then
      begin
         Info:= _doData('');
         _hi_onEvent(_event_onIndexStopAction,0);
         Exit;
      end;
   dt.data_type:= data_null;
   fdir:= ReadString(dt,_data_ElementsDir,_prop_ElementsDir);
   p := GetStartDir + 'Elements\' + fdir + '\code\' + fn;
   if not FileExists(p) then
      begin
         Info:= _doData(p);
         _hi_onEvent(_event_onIndexStopAction,1);
          Exit;
      end;
   list:= NewStrList;
   list.LoadFromFile(p);
   i:= 0;
   linkf:= '';
   while i < List.Count do
      begin
         if Copy(List.Items[i],1,4) = 'uses' then
            begin
               str:= '';
               while str <> ';' do
                  begin
                      linkf:= linkf + List.Items[i];
                      Replace(linkf,'uses','');
                      Replace(linkf,' ','');
                      spos:= pos('{', linkf);
                      while spos > 0 do
                         begin
                            epos:= pos('}', linkf);
                            Delete(linkf, spos, epos-spos+1);
                            spos:= pos('{', linkf);
                         end;
                      inc(i);
                      if linkf <> '' then str:= linkf[Length(linkf)];
                  end;
            end;
         inc(i);
      end;
   Replace(linkf,';',',');
   str:= GetTok(linkf,',');
   k:= 0;
   linklist:= NewStrList;
   while str <> '' do
      begin
         inc(k);
         Info:= _doData(k);
         linklist.Add(str+'.pas');
         _hi_onEvent(_event_onLinkFiles,str);
         str:= GetTok(linkf,',');
      end;

   ListLink.Add(fn);
   fd:= GetStartDir + 'Elements\' + fdir + '\code\';
   fn:= ExtractFileNameWOext(fn);
   str:= fn + '.h';
   if FileExists(fd + str) then ListLink.Add(str);
   str:= fn + '.cpp';
   if FileExists(fd + str) then ListLink.Add(str);
   Delete(fn,1,2);
   str:=fn + '.ico';
   if FileExists(GetStartDir + 'Elements\' + fdir + '\icon\' + str) then ListLink.Add(str);
   str:=fn + '.ini';
   if FileExists(GetStartDir + 'Elements\' + fdir + '\conf\' + str) then ListLink.Add(str);
   str:=fn + '.html';
   if FileExists(GetStartDir + 'Help\' + str) then ListLink.Add(str);
   fexcl:=ReadString(dt,_data_Exclusion,_prop_Exclusion);
   p:= kol.GetStartDir + fexcl;
   if FileExists(p) then
      begin
         list.clear;
         list.LoadFromFile(p);
         i:=0;
          while i < linklist.Count do
            begin
               notItem:=True;
               j:=0;
               while j < list.Count do
                  begin
                     if LowerCase(linklist.Items[i]) = LowerCase(list.Items[j]) then notItem:= False;
                     inc(j);
                  end;
               if notItem and FileExists(GetStartDir + 'Elements\' + fdir + '\code\' + linklist.Items[i]) then
                   ListLink.Add(linklist.Items[i]);
               inc(i);
            end;
      end
   else
      begin
         Info:= _doData(p);
         _hi_onEvent(_event_onIndexStopAction,4);
         i:=0;
         while i < linklist.Count do
            begin
               if FileExists(GetStartDir + 'Elements\' + fdir + '\code\' + linklist.Items[i]) then
                   ListLink.Add(linklist.Items[i]);
               inc(i);
            end;
      end;

   k:=0;
   i:=0;
   while  i < ListLink.Count do
      begin
         inc(k);
         Info:= _doData(k);
         _hi_onEvent(_event_onEndLinkFiles,ListLink.Items[i]);
         inc(i);
      end;
   linklist.free;
   list.free;
end;

//#####################################################################
//#                                                                   #
//#                                              #
//#                                                                   #
//#####################################################################

procedure ThiInstallEl._work_doExtractComp;
var
   dt:TData;
   flHIC,flHUC,list:PStrList;
   j,i,k,cnt:word;
   fn,CDir,EDir,Prf,Dfc,Tfc,Bfc,NBfc,Tab,Hint,Name,fdir,p,nm:string;
   notName:boolean;
begin
   Name:= ToString(_Data);
   dt.data_type:= data_null;
   Prf:= ReadString(dt,_data_PrefExtrComp,_prop_PrefExtrComp);
   CDir:= ReadString(dt,_data_DirComp,_prop_DirComp);
   EDir:= ReadString(dt,_data_ExtractDir,_prop_ExtractDir);
   fdir:= ReadString(dt,_data_ElementsDir,_prop_ElementsDir);
   flHIC := NewStrList;
   flHUC := NewStrList;

   p := GetStartDir + 'Elements\' + fdir + '\Elements.ini';

   if not FileExists(p) then
      begin
         inc(j);
         Info:= _doData(p);
         _hi_onEvent(_event_onIndexStopAction,1);
      end;

   list:= NewStrList;
   list.LoadFromFile(p);
   notName:= True;
   i:= 0;
   while i < list.Count do
      begin
         if Name + ',' = Copy(List.Items[i],1,Length(name)+1) then
            begin
               Hint:= Copy(List.Items[i],Length(Name)+2,Length(List.Items[i])-Length(Name)+2);
               notName:= False;
               break;
            end;
         inc(i);
      end;

   if notName then
      begin
         Info:=_doData(Name);
         _hi_onEvent(_event_onIndexStopAction,5);
      end;
   while (List.Items[i] <> '') and (i >= 0) do dec(i);
   while (Copy(List.Items[i],1,1) <> '+') and (i < list.Count) do inc(i);
   Tab:= Copy(List.Items[i],2, pos('|',List.Items[i])-2);
   flHIC.text:= Name + ',' + Tab + ',' + Hint + ',' + fdir;
   flHUC.text:= Name + ',' + fdir;
   ForceDirectories(GetStartDir + CDir);
   ForceDirectories(GetStartDir + CDir + '\' + EDir);
   ForceDirectories(GetStartDir + CDir + '\' + EDir + '\' + Prf + Name);
   flHIC.SaveToFile(GetStartDir + CDir + '\' + EDir + '\' + Prf + Name + '\' + Name + '.hic');
   flHUC.SaveToFile(GetStartDir + CDir + '\' + EDir + '\' + Prf + Name + '\' + Name + '.huc');

   list.free;
   flHIC.free;
   flHUC.free;

   fn:= 'hi' + Name + '.pas';
   dt.data_type:= data_str;
   dt.sdata:= fn;
   _work_doLinkFiles(dt,0);

   for k:=0 to ListLink.Count - 1 do
      begin
         Dfc:= GetStartDir + CDir + '\' + EDir + '\' + Prf + Name + '\' + ListLink.Items[k];
         if ExtractFileExt(ListLink.Items[k]) = '.pas' then
         Tfc:= GetStartDir + 'Elements\' + fdir + '\code\'
         else if ExtractFileExt(ListLink.Items[k]) = '.ico' then
         Tfc:= GetStartDir + 'Elements\' + fdir + '\icon\'
         else if ExtractFileExt(ListLink.Items[k]) = '.ini' then
         Tfc:= GetStartDir + 'Elements\' + fdir + '\conf\'
         else if ExtractFileExt(ListLink.Items[k]) = '.h' then
         Tfc:= GetStartDir + 'Elements\' + fdir + '\code\'
         else if ExtractFileExt(ListLink.Items[k]) = '.cpp' then
         Tfc:= GetStartDir + 'Elements\' + fdir + '\code\'
         else if ExtractFileExt(ListLink.Items[k]) = '.html' then
         Tfc:= GetStartDir + 'Help\';
         Tfc:= Tfc + ListLink.Items[k];

         CopyFile(PChar(Tfc),PChar(Dfc),true);
      end;

   Info:= _doData(ListLink.Count);
   _hi_onEvent(_event_onIndexStopAction,6);

end;

end.
